// Author: John Boyer
// Date  : 26 Dec 1997
 
import java.io.*;
import java.lang.*;
import java.util.*;

//==========================================================================
// Normally one should uses a linked list rather than ExtLinkedList
// This was only added to overcome the toArray() bug that appeared
// in the JDKv1.2b2.  The Collections.sort() function in that
// version sorts a linked list by building an array of references,
// sorting the array, then copying the references back into the
// linked list.  However, the initial conversion to an array was
// O(n^2) time due to a bug that kept restarting the iterator every
// time next() was called.
//==========================================================================

class ExtLinkedList extends LinkedList
{
        public Iterator iterator() 
        {
	    return listIterator();
        }
}

//==========================================================================
// START OF CLASSES RELATED TO MYLINKEDLIST
//==========================================================================

abstract class Node implements Comparable
{
	public Node next;
	public abstract int compareTo(Object obj);
}

class NodePair
{
	public Node first, last;
	public NodePair() { first=null; last=null; }
}

class DataNode extends Node
{
	private String key;

	public  int Pos; // Mock satellite data

	public DataNode() { key=null; }
	public DataNode(StringBuffer S) { key = S.toString(); }
	public DataNode(char S[]) { key = new String(S); }
	public String GetValue() { return key; }
	public void   SetValue(String S) { key=S; }
	public int compareTo(Object obj)
	{
		return key.compareTo(((DataNode) obj).key);
	}
}

class MyLinkedList
{
	private Node First;
	private int  NumNodes;
	
	public MyLinkedList()
	{
		First = null;
		NumNodes = 0;
	}

	public Node GetFirst() { return First; }
	public int  size() { return NumNodes; }

	//===============================================================
	// We use this function to get linear time when building the
	// linked list.  The building function keeps track of the last
	// element added.
	//===============================================================
	public Node AddAfter(Node predecessor, Node toAdd)
	{
		if (predecessor == null)
		{
			toAdd.next = First;
			First = toAdd;
		}
		else
		{
			toAdd.next = predecessor.next;
			predecessor.next = toAdd;
		}

		NumNodes++;
		return toAdd;
	}

	//===================================================================
	// THE LINKED LIST BINARY SEARCH
	// This binary search is very fast even though it typically does
	// twice as many node hops as the average sequential search because 
	// it only does O(log n) comparisons, which are virtually always
	// more costly in practice.
	//===================================================================
	public Node binarySearch(Object SearchKey)
	{
	Node PartitionFirst=First, MidPtr=null;
	int  PartitionSize=NumNodes, Mid, I, Result;

		while (PartitionSize > 0)
		{
			Mid = PartitionSize / 2;
			MidPtr = PartitionFirst;
			for (I=0; I < Mid; I++)
				MidPtr = MidPtr.next;

			Result = MidPtr.compareTo(SearchKey);

			if (Result > 0)
				PartitionSize = Mid;
			else if (Result < 0)
			{
				PartitionSize -= Mid;
				PartitionFirst = MidPtr;
			}
			else return MidPtr;			
		}
		return null;
	}

        //==============================================================
	// THE LINKED LIST MERGE SORT (recursive)
	// It is not necessary to do the merge sort non-recursively
	// because it is guaranteed to use only O(lg n) stack.
	// This version is much more understandable and faster.
	//==============================================================
	private void mergesort(Node before, Node F1, int N1, NodePair NP)
	{
		if (N1 <= 1)
			NP.first = NP.last = F1;
		else
		{
		Node F2;
		int N2; 

			N2 = N1; N1 >>= 1; N2 -= N1;
			mergesort(before, F1, N1, NP);
			F1 = NP.first;
			F2 = NP.last.next;
			mergesort(NP.last, F2, N2, NP);
			F2 = NP.first;
			merge(before, F1, N1, F2, N2, NP);
		}
	}

	//==============================================================      
	// This merge is used by the recursive merge sort
	// The sort will be stable provided that F1 receives the list
	// containing elements that occured earlier in the original list
	//==============================================================
	private void merge(Node before, Node F1, int N1, Node F2, int N2, NodePair NP)
	{
	Node first=null, last=null, temp=null;
	int I, J;
	
		first = last = F1.compareTo(F2) <= 0 ? F1 : F2;
		for (I=J=0; I < N1 || J < N2; )
		{
			if (I < N1 && (J>=N2 || F1.compareTo(F2) <= 0))
			     { temp = F1; F1 = F1.next; I++; }
			else { temp = F2; F2 = F2.next;	J++; }

			last.next = temp;
			last = temp;
		}

		if (before == null)
		     First = first;
		else before.next = first;
		last.next = F2;

		NP.first = first;
		NP.last = last;		
	}

	//==============================================================      
	// Here is a non-recursive merge sort that is very simple.
	// It begins by merging all lists of size 2, then 4, 8, etc.
	// It is a little slower than the recursive version due to 
	// the extra work of the k-loop.
	// This function is not called on in the main demo.
	//==============================================================      

	private void nr2_mergesort()
	{
	int i, j, k, N1, N2;
	Node F1, F2, before;
	NodePair NP = new NodePair();

		for (i=1; i < NumNodes; i<<=1)
		     for (before=null, N1=N2=i, j=0; j+N1<NumNodes; j+=i<<1)
		     {
			  F1 = F2 = before==null ? First : before.next;
			  for (k=0; k<N1; k++) 
			       F2 = F2.next;
			  if (N2 > NumNodes-j-N1) 
			      N2 = NumNodes-j-N1;		
			  merge(before, F1, N1, F2, N2, NP);
			  before = NP.last;
		     }
	}

	//=================================================================
	// THE LINKED LIST MERGE SORT (non-recursive)
	// This non-recursive merge sort was inspired by the Fibonacci heap
	// consolidation operation (thanks Fredman and Tarjan!).
	// If the binary representation of NumNodes has a 1 in bit i, then
	// Heads[i] will have 2^i nodes in it when we finish the main loop.
	// Then, a cleanup loop merges all of the partial lists together.
	// This algorithm is more advanced technically, yet it is slower
	// than the recursive version.
	//=================================================================
	
	// If you change this to 64, then change all ints to longs
	private final static int NUMMACHINEBITS = 32;

	private void nr_mergesort()
	{
	MyLinkedList[] Heads = new MyLinkedList[NUMMACHINEBITS];
	Node Current;
	int  i, Pow2;

		for (i=0; i<NUMMACHINEBITS; i++)
			Heads[i] = new MyLinkedList();

		while (NumNodes > 0)
		{
			Current = First;
			First = First.next;
			NumNodes--;
			Current.next = null;

			Heads[0].First = merge(Heads[0].First, Heads[0].NumNodes, Current, 1);
			Heads[0].NumNodes++;

			for (i=0,Pow2=1; Heads[i].NumNodes>Pow2; i++,Pow2<<=1)
			{
				Heads[i+1].First = merge(Heads[i+1].First, Heads[i+1].NumNodes,
						         Heads[i].First, Heads[i].NumNodes);
				Heads[i+1].NumNodes += Heads[i].NumNodes;
				Heads[i].First = null;
				Heads[i].NumNodes = 0;
			}
		}

		for (i=0; i<NUMMACHINEBITS; i++)
		{
			First = merge(Heads[i].First, Heads[i].NumNodes, First, NumNodes);
			NumNodes += Heads[i].NumNodes;
		}
	}

	//==============================================================      
	// This merge is used by the non-recursive merge sort.
	// The sort will be stable provided that F1 receives the list
	// containing elements that occured earlier in the original list
	//==============================================================
	private Node merge(Node F1, int N1, Node F2, int N2)
	{
	Node first=null, last=null, temp=null;
	int I, J;

		if (F1 == null) return F2;
		else if (F2 == null) return F1;
	
		first = last = F1.compareTo(F2) <= 0 ? F1 : F2;
		for (I=J=0; I < N1 || J < N2; )
		{
			if (I < N1 && (J>=N2 || F1.compareTo(F2) <= 0))
			     { temp = F1; F1 = F1.next; I++; }
			else { temp = F2; F2 = F2.next;	J++; }

			last.next = temp;
			last = temp;
		}
		last.next = null;
		return first;
	}

	//===================================================================
	//===================================================================
	private void PrintDebugInfo(Node before, Node first, int n, Node pivot, int Num1, int Num2)
	{
		System.out.println("before=" + (before==null ? "null" : ((DataNode) before).GetValue()));
		System.out.println("pivot=" + (pivot==null ? "null" : ((DataNode) pivot).GetValue()));
		System.out.println("n=" +n+ ",Num1=" +Num1+ ",Num2=" +Num2);
		if (first==null)
			System.out.print("first=NULL");
		else
			for (int i=0; i<n; i++)
			{
				System.out.print( ((DataNode) first).GetValue() + " ");
				first = first.next;
			}

		System.out.print("\n\n");
	}

	//===================================================================
	//===================================================================
	private Node QuickSort(Node before, Node first, int n)
	{
	int Num1=0, Num2=n, i=1;
	Node Pivot=first, aNode=first, aNodePrev=first;

		// Pivot Advancement 
		for (i=1; i<n; i++, aNode=aNode.next)
		{
			if (aNode.compareTo(aNode.next) > 0)
				break;
			if ((i&1)==0)
			{
			    Pivot = Pivot.next;
			    Num1++;
			}
		}

		// Recognize sortedness in linear time
		if (i == n) return first;

		// Partition list by unlinking nodes with values less 
		// than the pivot and pushing them onto front of list
		for (aNodePrev = aNode; i < n; i++)
		{
			aNode = aNodePrev.next;
			if (Pivot.compareTo(aNode) > 0)
			{
				aNodePrev.next = aNode.next;
				aNode.next = first;
				first = aNode;
				Num1++;
			}
			else aNodePrev = aNode;
		}

		if (before!=null) before.next = first;
		Num2 = n - Num1 - 1;

		// Recurse to solve sublists
		if (Num1 > 1) first = QuickSort(before, first, Num1);
		if (Num2 > 1) QuickSort(Pivot, Pivot.next, Num2);
		
		return first;
	}

	//===================================================================
	/* Here is MEDIAN OF THREE Code.  To use, call with
			Pivot = aNode = MedianOf3(before,first,n);
		in lieu of the first loop of the QuickSort.
		It's actually faster to cut and paste this code into the
		quicksort, but since this code is far less elegant and takes 
		longer anyway... You can do it if you want to. */
	//===================================================================

	private Node MedianOf3(Node before, Node first, int n)
	{
	int i;
	Node Pivot, aNodePrev;

		if (n > 8)
		{
		int ndiv2 = (n>>1)+(n&1);
		Node mid=first, last=first;

			for (i=1; i<ndiv2; i++)
				mid = mid.next;
			for (i=ndiv2+1, last=mid; i<n; i++)
				last = last.next;

			Pivot = null;
			if (first.compareTo(mid.next) < 0)
			{
			     if (first.compareTo(last.next) < 0)
				 Pivot = mid.next.compareTo(last.next)<0 ? mid : last;
			}
			else if (first.compareTo(last.next) > 0)
				 Pivot = mid.next.compareTo(last.next)>0 ? mid : last;

			if (Pivot==null) Pivot = first;
			else
			{
				aNodePrev = Pivot;
				Pivot = Pivot.next;
				aNodePrev.next = Pivot.next;
				Pivot.next = first;
				first = Pivot;
			}
		}
		return first;
	}

	//===================================================================
	//===================================================================
	public void sort(char Choice)
	{
		if (Choice == 'M')
			mergesort(null, First, NumNodes, new NodePair());

		else if (Choice == 'N')
			nr_mergesort();

		else if (Choice == 'Q')
			First = QuickSort(null, First, NumNodes);

		else System.out.println("Undefined sort");
	}
}
//==========================================================================
// END OF CLASSES RELATED TO MYLINKEDLIST
//==========================================================================

//==========================================================================
// START OF CLASS SORTDEMO
//==========================================================================

class SortDemo
{
MyLinkedList MSortList, QSortList, NR_MSortList;
ExtLinkedList JDKSortList, BogusList;
DataNode[] JDKSortArray;
DataNode[] UnsortedArray;

// NOTE: It is fairly straightforward to change this program so that
// JDKSortList and JDKSortArray process strings instead of DataNodes.
// This would make them somewhat faster than they currently are, but
// I did not think it accurately reflected the most common linked lists,
// which usu. must create structures to associate key data with 
// satellite data.  For those who are curious, my first version of this
// test program directly used strings in JDKSortList and JDKSortArray.
// The results of the new sorts and the new binary search are still 
// dramatically faster.  Try it!

	//==================================================================
	// Constructor generates the strings
	//==================================================================

	//Uncomment to TEST STABILITY
	//private final static int STRINGSIZE = 2; 

	private final static int STRINGSIZE = 10;

      	public SortDemo(int N) 
      	{
	DataNode last1=null, last2=null, last3=null;
	char TempStr[] = new char[STRINGSIZE];
	Random RGen = new Random();
	int RandNum;

		MSortList = new MyLinkedList();
		QSortList = new MyLinkedList();
		NR_MSortList = new MyLinkedList();

		JDKSortList = new ExtLinkedList();
		BogusList = new ExtLinkedList();

		JDKSortArray = new DataNode[N];
		UnsortedArray = new DataNode[N];

		System.out.println("Building strings...");
		for (int i = 0; i < N; i++)
		{
			if (i%1000==0)
				System.out.print("\r"+i);
			RandNum = RGen.nextInt();
			for (int j=0; j < STRINGSIZE; j++, RandNum>>=1)
				TempStr[j] = (char) ('A'+(RandNum&15));

			last1 = (DataNode) MSortList.AddAfter(last1, new DataNode(TempStr));
			last2 = (DataNode) QSortList.AddAfter(last2, new DataNode(TempStr));
			last3 = (DataNode) NR_MSortList.AddAfter(last3, new DataNode(TempStr));

			JDKSortList.add(new DataNode(TempStr));
			if (i < 4000)
				BogusList.add(new DataNode(TempStr));

			JDKSortArray[i] = new DataNode(TempStr);
			UnsortedArray[i] = new DataNode(TempStr);
		}
		System.out.println("\r"+N);
      	}

	//==================================================================
	// Writes the strings in the array out to the given text file,
	// one number per line
	//==================================================================

      	public void WriteToFile(String FileName, MyLinkedList aList) 
	throws IOException
	{
	PrintWriter Outfile = new PrintWriter(
                              new BufferedWriter(
                              new FileWriter(FileName)));

	DataNode iterator = (DataNode) aList.GetFirst();

		for (int i = 0; i < aList.size(); i++)
		{
			Outfile.println(iterator.GetValue());	
			iterator = (DataNode) iterator.next;
		}
		if (iterator != null) System.out.println("iterator is not null");

		Outfile.close();
	}

	//==================================================================
	//==================================================================

      	public void SearchTest() 
	{
	long TimeBefore, SearchTestTime;
	DataNode aNode;

	//Uncomment to TEST STABILITY
	//	aNode = (DataNode) MSortList.GetFirst();
	//	for (int i=0; i < MSortList.size(); i++, aNode=(DataNode)aNode.next)
	//		aNode.Pos = i;

		// Run the bogus sort
		Sort('0');

	//==============================
	// Test new search
	//==============================
		MSortList.sort('N');

		System.out.print("New Search...");
		TimeBefore = System.currentTimeMillis();

		for (int i = 0; i < UnsortedArray.length; i++)
		{
			aNode = (DataNode) MSortList.binarySearch(UnsortedArray[i]);
			if (aNode == null)
				System.out.println("Error finding " + UnsortedArray[i].GetValue());
			//Uncomment to TEST STABILITY
			//if (SearchNode.compareTo(aNode)!=0)
			//	System.out.println("Error finding " + UnsortedArray[i].GetValue());
			//else    System.out.println("Found " + aNode.GetValue() + aNode.Pos);
		}
		
		SearchTestTime = System.currentTimeMillis() - TimeBefore;
		System.out.println("Done "+SearchTestTime);

	//==============================
	// Test Collections.binarySearch
	//==============================
		Collections.sort(JDKSortList);
		System.out.print("Collections.binarySearch...");
		TimeBefore = System.currentTimeMillis();

		for (int i = 0; i < UnsortedArray.length; i++)
		{
			if (Collections.binarySearch(JDKSortList, UnsortedArray[i]) < 0)
				System.out.println("Error finding " + UnsortedArray[i]);
		}
		
		SearchTestTime = System.currentTimeMillis() - TimeBefore;
		System.out.println("Done "+SearchTestTime);
	}

	//==================================================================
	//==================================================================

	private void Sort(char SortId)
	{
	long TimeBefore, SortTime;
	MyLinkedList aList = null;
	String SortName=null;

		switch (SortId)
		{
			case '0': SortName="Warmup sort"; break;
			case '1': SortName="Arrays.sort()"; break;
			case '2': SortName="Collections.sort()"; break;
			case '3': SortName="MSort"; aList = MSortList; break;
			case '4': SortName="QSort"; aList = QSortList; break;
			case '5': SortName="NR MSort"; aList = NR_MSortList; break;
		}

		System.out.print(SortName+" ");
		TimeBefore = System.currentTimeMillis();

		if (SortId == '0') Collections.sort(BogusList);
		else if (SortId == '1')	Arrays.sort(JDKSortArray);
		else if (SortId == '2')	Collections.sort(JDKSortList);
		else aList.sort(SortName.charAt(0));

		SortTime = System.currentTimeMillis() - TimeBefore;
		System.out.println(" Done " + SortTime);
	}

	//==================================================================
	//==================================================================

	public void SortTest()
	{
		// We sort the bogus list first to get Java to settle into the
		// idea that it's going to sort.  On small data sets, Java seems
		// to take longer on the first sort, whichever one we choose
		Sort('0');
		Sort('1');
		Sort('2');
		Sort('3');
		Sort('4');
		Sort('5');
	}
}

//=====================================================================
//=====================================================================

class Main
{
	public static void main(String args[]) 
	throws IOException
	{
	SortDemo SortObject = null;
	int N=16000;
	char Choice;
	BufferedReader Kbd = new BufferedReader(
                             new InputStreamReader(System.in), 1);

		System.out.println("1=Sort Test, 2=Search Test, n=New Size, ?=Help, x=Exit");

		do {
			System.out.print("Enter Choice:");
			Choice = (char) Kbd.readLine().charAt(0);
			System.out.println(Choice);

			System.gc();

			switch (Choice)
			{
				case '1':
				case '2':
					SortObject = new SortDemo(N);
					if (Choice=='1')
						SortObject.SortTest();
					else	SortObject.SearchTest();
					break;
				case 'n':
					System.out.print("How many strings do you want to sort?");
					N = Integer.parseInt(Kbd.readLine());
					System.out.println(N);
					break;
				case '?':
					System.out.println("1=Sort Test, 2=Search Test, n=New Size, ?=Help, x=Exit");
					break;
			}
			SortObject = null;
		} while (Choice != 'x');
	}
}

                                            
